io_m3_common.o: \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/disc_io/io_m3_common.c \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/disc_io.h \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_types.h

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/disc_io.h:

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_types.h:
