mb2print.o: /Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/mb2print.c \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/mbv2.h

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/mbv2.h:
