disc.o: /Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/disc_io/disc.c \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/disc_io.h \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_types.h \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/disc_io/io_mpcf.h \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/disc_io/io_m3cf.h \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/disc_io/io_sccf.h \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/disc_io/io_scsd.h \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/disc_io/io_m3sd.h

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/disc_io.h:

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_types.h:

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/disc_io/io_mpcf.h:

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/disc_io/io_m3cf.h:

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/disc_io/io_sccf.h:

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/disc_io/io_scsd.h:

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/disc_io/io_m3sd.h:
