BoyScout.o: \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/BoyScout/BoyScout.c \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/BoyScout.h \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/BoyScout/GBASoundRegs.h

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/BoyScout.h:

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/BoyScout/GBASoundRegs.h:
