xcomms_print.o: \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/xcomms_print.c \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_sio.h \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_base.h \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_types.h \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/xcomms_cmd.h \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/xcomms.h

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_sio.h:

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_base.h:

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_types.h:

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/xcomms_cmd.h:

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/xcomms.h:
