console.o: /Users/a3f/prjs/Device-GBA/share/boot2/libgba/src/console.c \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_video.h \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_base.h \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_types.h \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_systemcalls.h \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_console.h \
 /Users/a3f/prjs/Device-GBA/share/boot2/libgba/build/amiga_fnt.h

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_video.h:

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_base.h:

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_types.h:

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_systemcalls.h:

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/include/gba_console.h:

/Users/a3f/prjs/Device-GBA/share/boot2/libgba/build/amiga_fnt.h:
